@@ -8,6 +8,8 @@ typedef union
   struct { string_list_ty stringlist; lex_pos_ty pos; bool obsolete; } stringlist;
   struct { long number; lex_pos_ty pos; bool obsolete; } number;
   struct { lex_pos_ty pos; bool obsolete; } pos;
+  struct { char *ctxt; char *id; char *id_plural; lex_pos_ty pos; bool obsolete; } prev;
+  struct { char *prev_ctxt; char *prev_id; char *prev_id_plural; char *ctxt; lex_pos_ty pos; bool obsolete; } message_intro;
   struct { struct msgstr_def rhs; lex_pos_ty pos; bool obsolete; } rhs;
 } po_gram_stype;
 # define YYSTYPE po_gram_stype
@@ -16,13 +18,17 @@ typedef union
 # define	COMMENT	257
 # define	DOMAIN	258
 # define	JUNK	259
-# define	MSGCTXT	260
-# define	MSGID	261
-# define	MSGID_PLURAL	262
-# define	MSGSTR	263
-# define	NAME	264
-# define	NUMBER	265
-# define	STRING	266
+# define	PREV_MSGCTXT	260
+# define	PREV_MSGID	261
+# define	PREV_MSGID_PLURAL	262
+# define	PREV_STRING	263
+# define	MSGCTXT	264
+# define	MSGID	265
+# define	MSGID_PLURAL	266
+# define	MSGSTR	267
+# define	NAME	268
+# define	NUMBER	269
+# define	STRING	270
 
 
 extern DLL_VARIABLE YYSTYPE po_gram_lval;
