@@ -1,5 +1,5 @@
 /* xgettext C# backend.
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005-2006 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -33,6 +33,8 @@
 #include "error.h"
 #include "error-progname.h"
 #include "xalloc.h"
+#include "xerror.h"
+#include "xvasprintf.h"
 #include "exit.h"
 #include "hash.h"
 #include "po-charset.h"
@@ -74,26 +76,20 @@ x_csharp_keyword (const char *name)
   else
     {
       const char *end;
-      int argnum1;
-      int argnum2;
+      struct callshape shape;
       const char *colon;
 
       if (keywords.table == NULL)
-	init_hash (&keywords, 100);
+	hash_init (&keywords, 100);
 
-      split_keywordspec (name, &end, &argnum1, &argnum2);
+      split_keywordspec (name, &end, &shape);
 
       /* The characters between name and end should form a valid C#
 	 identifier sequence with dots.
 	 A colon means an invalid parse in split_keywordspec().  */
       colon = strchr (name, ':');
       if (colon == NULL || colon >= end)
-	{
-	  if (argnum1 == 0)
-	    argnum1 = 1;
-	  insert_entry (&keywords, name, end - name,
-			(void *) (long) (argnum1 + (argnum2 << 10)));
-	}
+	insert_keyword_callshape (&keywords, name, end - name, &shape);
     }
 }
 
@@ -104,6 +100,8 @@ init_keywords ()
 {
   if (default_keywords)
     {
+      /* When adding new keywords here, also update the documentation in
+	 xgettext.texi!  */
       x_csharp_keyword ("GetString");	/* Resource{Manager,Set}.GetString */
       x_csharp_keyword ("GetPluralString:1,2");	/* GettextResource{Manager,Set}.GetPluralString */
       default_keywords = false;
@@ -230,7 +228,12 @@ Please specify the source encoding through --from-code.\n"),
 	 through phase1_getc as needed.  This is needed to give reasonable
 	 interactive behaviour when fp is connected to an interactive tty.  */
       unsigned char buf[MAX_PHASE1_PUSHBACK];
-      size_t bufcount = 0;
+      size_t bufcount;
+      int c = phase1_getc ();
+      if (c == EOF)
+	return UEOF;
+      buf[0] = (unsigned char) c;
+      bufcount = 1;
 
       for (;;)
 	{
@@ -1859,25 +1862,19 @@ static flag_context_list_table_ty *flag_context_list_table;
 /* Extract messages until the next balanced closing parenthesis or brace,
    depending on TERMINATOR.
    Extracted messages are added to MLP.
-   When a specific argument shall be extracted, COMMAS_TO_SKIP >= 0 and,
-   if also a plural argument shall be extracted, PLURAL_COMMAS > 0,
-   otherwise PLURAL_COMMAS = 0.
-   When no specific argument shall be extracted, COMMAS_TO_SKIP < 0.
    Return true upon eof, false upon closing parenthesis or brace.  */
 static bool
 extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 		       flag_context_ty outer_context,
 		       flag_context_list_iterator_ty context_iter,
-		       int commas_to_skip, int plural_commas)
+		       struct arglist_parser *argparser)
 {
-  /* Remember the message containing the msgid, for msgid_plural.  */
-  message_ty *plural_mp = NULL;
-
+  /* Current argument number.  */
+  int arg = 1;
   /* 0 when no keyword has been seen.  1 right after a keyword is seen.  */
   int state;
   /* Parameters of the keyword just seen.  Defined only in state 1.  */
-  int next_commas_to_skip = -1;
-  int next_plural_commas = 0;
+  const struct callshapes *next_shapes = NULL;
   /* Context iterator that will be used if the next token is a '('.  */
   flag_context_list_iterator_ty next_context_iter =
     passthrough_context_list_iterator;
@@ -1943,15 +1940,11 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 	      {
 		void *keyword_value;
 
-		if (find_entry (&keywords, dottedname, strlen (dottedname),
-				&keyword_value)
+		if (hash_find_entry (&keywords, dottedname, strlen (dottedname),
+				     &keyword_value)
 		    == 0)
 		  {
-		    int argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
-		    int argnum2 = (int) (long) keyword_value >> 10;
-
-		    next_commas_to_skip = argnum1 - 1;
-		    next_plural_commas = (argnum2 > argnum1 ? argnum2 - argnum1 : 0);
+		    next_shapes = (const struct callshapes *) keyword_value;
 		    state = 1;
 		    break;
 		  }
@@ -1988,16 +1981,26 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 	case token_type_lparen:
 	  if (extract_parenthesized (mlp, token_type_rparen,
 				     inner_context, next_context_iter,
-				     state ? next_commas_to_skip : -1,
-				     state ? next_plural_commas : 0))
-	    return true;
+				     arglist_parser_alloc (mlp,
+							   state ? next_shapes : NULL)))
+	    {
+	      xgettext_current_source_encoding = po_charset_utf8;
+	      arglist_parser_done (argparser, arg);
+	      xgettext_current_source_encoding = xgettext_global_source_encoding;
+	      return true;
+	    }
 	  next_context_iter = null_context_list_iterator;
 	  state = 0;
 	  continue;
 
 	case token_type_rparen:
 	  if (terminator == token_type_rparen)
-	    return false;
+	    {
+	      xgettext_current_source_encoding = po_charset_utf8;
+	      arglist_parser_done (argparser, arg);
+	      xgettext_current_source_encoding = xgettext_global_source_encoding;
+	      return false;
+	    }
 	  if (terminator == token_type_rbrace)
 	    {
 	      error_with_progname = false;
@@ -2013,15 +2016,25 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 	case token_type_lbrace:
 	  if (extract_parenthesized (mlp, token_type_rbrace,
 				     null_context, null_context_list_iterator,
-				     -1, 0))
-	    return true;
+				     arglist_parser_alloc (mlp, NULL)))
+	    {
+	      xgettext_current_source_encoding = po_charset_utf8;
+	      arglist_parser_done (argparser, arg);
+	      xgettext_current_source_encoding = xgettext_global_source_encoding;
+	      return true;
+	    }
 	  next_context_iter = null_context_list_iterator;
 	  state = 0;
 	  continue;
 
 	case token_type_rbrace:
 	  if (terminator == token_type_rbrace)
-	    return false;
+	    {
+	      xgettext_current_source_encoding = po_charset_utf8;
+	      arglist_parser_done (argparser, arg);
+	      xgettext_current_source_encoding = xgettext_global_source_encoding;
+	      return false;
+	    }
 	  if (terminator == token_type_rparen)
 	    {
 	      error_with_progname = false;
@@ -2035,19 +2048,7 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 	  continue;
 
 	case token_type_comma:
-	  if (commas_to_skip >= 0)
-	    {
-	      if (commas_to_skip > 0)
-		commas_to_skip--;
-	      else
-		if (plural_mp != NULL && plural_commas > 0)
-		  {
-		    commas_to_skip = plural_commas - 1;
-		    plural_commas = 0;
-		  }
-		else
-		  commas_to_skip = -1;
-	    }
+	  arg++;
 	  inner_context =
 	    inherited_context (outer_context,
 			       flag_context_list_iterator_advance (
@@ -2062,45 +2063,16 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 	    pos.file_name = logical_file_name;
 	    pos.line_number = token.line_number;
 
+	    xgettext_current_source_encoding = po_charset_utf8;
 	    if (extract_all)
-	      {
-		xgettext_current_source_encoding = po_charset_utf8;
-		savable_comment_to_xgettext_comment (token.comment);
-		remember_a_message (mlp, token.string, inner_context, &pos);
-		savable_comment_reset ();
-		xgettext_current_source_encoding = xgettext_global_source_encoding;
-	      }
+	      remember_a_message (mlp, NULL, token.string, inner_context,
+				  &pos, token.comment);
 	    else
-	      {
-		if (commas_to_skip == 0)
-		  {
-		    if (plural_mp == NULL)
-		      {
-			/* Seen an msgid.  */
-			message_ty *mp;
-
-			xgettext_current_source_encoding = po_charset_utf8;
-			savable_comment_to_xgettext_comment (token.comment);
-			mp = remember_a_message (mlp, token.string,
-						 inner_context, &pos);
-			savable_comment_reset ();
-			xgettext_current_source_encoding = xgettext_global_source_encoding;
-			if (plural_commas > 0)
-			  plural_mp = mp;
-		      }
-		    else
-		      {
-			/* Seen an msgid_plural.  */
-			xgettext_current_source_encoding = po_charset_utf8;
-			remember_a_message_plural (plural_mp, token.string,
-						   inner_context, &pos);
-			xgettext_current_source_encoding = xgettext_global_source_encoding;
-			plural_mp = NULL;
-		      }
-		  }
-		else
-		  free (token.string);
-	      }
+	      arglist_parser_remember (argparser, arg, token.string,
+				       inner_context,
+				       pos.file_name, pos.line_number,
+				       token.comment);
+	    xgettext_current_source_encoding = xgettext_global_source_encoding;
 	  }
 	  drop_reference (token.comment);
 	  next_context_iter = null_context_list_iterator;
@@ -2108,6 +2080,9 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 	  continue;
 
 	case token_type_eof:
+	  xgettext_current_source_encoding = po_charset_utf8;
+	  arglist_parser_done (argparser, arg);
+	  xgettext_current_source_encoding = xgettext_global_source_encoding;
 	  return true;
 
 	case token_type_dot:
@@ -2150,7 +2125,7 @@ extract_csharp (FILE *f,
      due to an unbalanced closing parenthesis, just restart it.  */
   while (!extract_parenthesized (mlp, token_type_eof,
 				 null_context, null_context_list_iterator,
-				 -1, 0))
+				 arglist_parser_alloc (mlp, NULL)))
     ;
 
   fp = NULL;
